@@ -63,11 +63,11 @@ pub enum V1Enemies {
 pub struct Spawn<SpawnType> {
     pub enemy_type: SpawnType,
     pub delay: f32,
-    _u1: u32,
-    _u2: u32,
-    _u3: u32,
-    _u4: u32,
-    _u5: u32
+    pub _u1: u32,
+    pub _u2: u32,
+    pub _u3: u32,
+    pub _u4: u32,
+    pub _u5: u32
 }
 
 #[repr(C)]
@@ -80,8 +80,8 @@ pub struct Header {
     pub shrink_rate: f32,
     pub brightness: f32,
     pub game_mode: i32,
-    _u1: u32,
-    _u2: u32,
+    pub _u1: u32,
+    pub _u2: u32,
 }
 
 #[repr(C)]
@@ -93,15 +93,15 @@ pub struct Arena {
 #[repr(C)]
 #[derive(Debug)]
 pub struct SpawnsHeader {
-    _u1: u32,
-    _u2: u32,
-    _u3: u32,
-    _u4: u32,
+    pub _u1: u32,
+    pub _u2: u32,
+    pub _u3: u32,
+    pub _u4: u32,
     pub devil_dagger_time: i32,
     pub gold_dagger_time: i32,
     pub silver_dagger_time: i32,
     pub bronze_dagger_time: i32,
-    _u5: u32,
+    pub _u5: u32,
     pub spawn_count: i32,
 }
 
@@ -112,6 +112,104 @@ pub struct Settings {
     pub timer_start: Option<f32>,
 }
 
+///////////////
+/*** IMPLS ***/
+///////////////
+
+impl std::default::Default for V3Enemies {
+    fn default() -> Self {
+        V3Enemies::Empty
+    }
+}
+
+impl std::default::Default for V2Enemies {
+    fn default() -> Self {
+        V2Enemies::Empty
+    }
+}
+
+impl std::default::Default for V1Enemies {
+    fn default() -> Self {
+        V1Enemies::Empty
+    }
+}
+
+impl<T: Default> std::default::Default for Spawn<T> {
+    fn default() -> Self {
+        Spawn {
+            enemy_type: T::default(),
+            delay: 0.0,
+            _u1: 0,
+            _u2: 3,
+            _u3: 0,
+            _u4: 1106247680,
+            _u5: 10,
+        }
+    }
+}
+
+impl std::default::Default for Header {
+    fn default() -> Self {
+        Header {
+            spawn_version: 6,
+            world_version: 9,
+            shrink_end_radius: 20.,
+            shrink_start_radius: 50.,
+            shrink_rate: 0.025,
+            brightness: 60.,
+            game_mode: 0,
+            _u1: 51,
+            _u2: 1,
+        }
+    }
+}
+
+impl std::default::Default for Arena {
+    fn default() -> Self {
+        Arena {
+            data: [-1000.; 51*51],
+        }
+    }
+}
+
+// Accessors 2D -> 1D
+impl Arena {
+    pub fn get_tile(&self, x: u16, y: u16) -> &f32 {
+        &self.data[y as usize * 51 + x as usize]
+    }
+
+    pub fn get_tile_mut(&mut self, x: u16, y: u16) -> &mut f32 {
+        &mut self.data[y as usize * 51 + x as usize]
+    }
+}
+
+impl std::default::Default for SpawnsHeader {
+    fn default() -> Self {
+        SpawnsHeader {
+            devil_dagger_time: 500,
+            gold_dagger_time: 250,
+            silver_dagger_time: 120,
+            bronze_dagger_time: 60,
+            spawn_count: 0,
+            _u1: 0,
+            _u2: 0,
+            _u3: 0,
+            _u4: 1,
+            _u5: 0,
+        }
+    }
+}
+
+impl std::default::Default for Settings {
+    fn default() -> Self {
+        Settings {
+            additional_gems: 0,
+            initial_hand: 0,
+            timer_start: Some(0.0),
+        }
+    }
+}
+
 impl<SpawnType: Clone> Spawnset<SpawnType> {
     pub unsafe fn deserialize<R: Read>(source: &mut R) -> Result<Self> {
         let mut header: Header = std::mem::zeroed();
@@ -174,7 +272,12 @@ impl<SpawnType: Clone> Spawnset<SpawnType> {
                 }
             }
         }
+        sink.flush()?;
         Ok(())
     }
+
+    pub fn recalculate_spawn_count(&mut self) {
+        self.spawns_header.spawn_count = self.spawns.len() as i32;
+    }
 }
 
